#set working directory to data folder
setwd("~/surfdrive/Documents/Y2/Papers/JMIR/JMIR_TechSOA_EHealth_Depression/Analyses_TechSOA_EHealth_Depression")

#save output to file
sink("output/output-analysis_jmir.txt")
sessionInfo()

cat("#########################################################################################################\n")
cat("# Analysis of technological state of the art in e-mental health for the prevention and treatment of depression\n") 
cat("# 2019 TUDelft\n")
cat("# Author: Franziska Burger\n")
cat("# Date: September 2018\n")
cat("# Update: August 2019\n")
cat("# Final examination for correspondence with paper done by: Franziska Burger and Willem-Paul Brinkman")
cat("# Output files: \n")
cat("#  1. output-analysis_jmir.txt\n")
cat("#  2. figure[3-9].png\n")
cat("#  3. mma10.png\n")
cat("#  4. mma7[a-b].csv\n")
cat("# Required:\n")
cat("#  1. ehealth4mdd database (must be loaded in mysql server)\n")
cat("#  2. ehealth4mdd_CYPSY23 database (must be loaded in mysql server)\n")
cat("#  3. function_classification_C1C3.txt\n")
cat("#  4. function_identification_C1C3.txt\n")
cat("#  5. function_therapy_mapping_C1C3.txt\n")
cat("#  6. literature_screening_C1C2.txt\n")
cat("#  7. function_identification_C1C3\n")

cat("# #########################################################################################################\n")
cat("\n")
cat("\n")
cat("################################# LIBRARIES #################################\n")
require(car)
require(plyr)
require(dplyr)
require(stringr)
require(tidyr)
require(reshape)
require(reshape2)
require(ggplot2)
require(RMySQL)
require(magrittr)
require(psych)
require(relen)
require(Hmisc)
require(gplots)
require(dendextend)
require(colorspace)
require(pheatmap)
require(gridExtra)
require(grid)
require(cowplot)
require(coin)
require(nlme)
require(ggrepel)
require(extrafont)
require(irr)
require(boot)
require(rcompanion)
cat("# car\n
    # plyr\n
    # dplyr\n
    # stringr\n
    # tidyr\n
    # reshape\n
    # reshape2\n
    # ggplot2\n
    # RMySQL\n
    # magrittr\n
    # psych\n
    # relen\n
    # Hmisc\n
    # gplots\n
    # dendextend\n
    # colorspace\n
    # pheatmap\n
    # gridExtra\n
    # grid\n
    # cowplot\n
    # coin\n
    # nlme\n
    # ggrepel\n
    # extrafont\n
    # irr\n
    # boot\n
    # rcompanion\n")
cat("#################################")
cat("\n")
cat("\n")

########################## FUNCTIONS
#some versions have multiple therapy types or technologies separated by commas or semicolons, here we split these up and create new rows
split_list <- function(df,column,ids){
  #remove spaces
  df[,c(column)] <- lapply(df[,c(column)], function(x){gsub(" ","",x)}) %>% unlist(.)
  #find length of string with most elements (counting commas/semicolons), e.g. '1,8,9' is longer than '7,15'
  n <- df[,c(column)] %>% lapply(function(x){ifelse(is.na(x),0,str_count(x,",|;"))}) %>% unlist(.) %>% max(.)
  n <- n+1
  #expand string into new columns
  df.split <- separate_(df, col=column, into=as.character(c(1:n)), sep=",|;")
  #melt dataframe again so that it has the new columns as new rows
  df.melted <- melt(df.split[,c(ids,as.character(c(1:n)))],id=ids,na.rm=TRUE)
  df.melted$variable <- NULL
  df.melted$value <- factor(df.melted$value)
  return(df.melted)
}

#replace old values in a dataframe column with new values
replace_vals <- function(dfcol, old_vals, new_vals){
  for (i in 1:length(old_vals)){
    dfcol <- replace(dfcol, which(dfcol %in% old_vals[[i]]), new_vals[[i]])
  }
  return(as.factor(dfcol))
}

#counts of number of systems or versions that exhibit a certain feature
summary_statistics <- function(df,feature,ids,groupvars,old_vals,new_vals){
  df.new <- split_list(df,feature,ids)
  if(!missing(old_vals) && !missing(new_vals)){
    #df.new$value <- replace_vals(as.numeric(as.character(df.new$value)),old_vals,new_vals)
    df.new$value <- replace_vals(as.character(df.new$value),old_vals,new_vals)
  }
  #we can also look at the number of versions that have a certain feature
  vcount <- df.new$VersionKey %>% unique(.) %>% length(.)
  nvwithfeat <- df.new[,c("VersionKey","value")] %>% unique(.) %>% group_by(value) %>% summarise(.,count=n(),perc=count/vcount)
  #or the number of systems that have a certain feature
  scount <- df.new$SystemKey %>% unique(.) %>% length(.)
  nswithfeat <- df.new[,c("SystemKey","value")] %>% unique(.) %>% group_by(value) %>% summarise(.,count=n(), perc=count/scount)
  return(list("nvc"=nvwithfeat, "nsc"=nswithfeat))
}

#get all rows of df.g1 that are of a specific function type
ehdts_subdf <- function(functype){
  subdf <- df.g1[which(df.g1$FunctionSubType==functype),c("VersionKey","eHDTS")] %>% group_by(VersionKey) %>% dplyr::summarise(., fcount=n(),tsmean=mean(eHDTS))
  boxplot(subdf$tsmean,main=functype)
  return(subdf)
}

#Snijder & Bosker R^2 (p.112) (see Finch, Bolin, & Kelley p.49)  for multilevel models for Level 1
getRSquaredLevel1 <- function(Model0, Model1){
  #Variances
  vc.Model0 <- VarCorr(Model0)
  sigma0 <-Model0$sigma
  tau0 <- as.numeric(vc.Model0[1,2])
  # tau0 <- as.numeric(vc.Model0[4,2]) would be for nested effect, vc.model[row, colomn], refers to table 
  sigma0SQ <- sigma0 * sigma0
  tau0SQ <- tau0 * tau0
  vc.Model1 <- VarCorr(Model1)
  sigma1 <-Model1$sigma
  tau1 <- as.numeric(vc.Model1[1,2])
  sigma1SQ <- sigma1 * sigma1
  tau1SQ <- tau1 * tau1
  Rsquare <- 1 - ((sigma1SQ + tau1SQ) / (sigma0SQ + tau0SQ))
  return(Rsquare)
}

#based on Finch, Bolin and Kelley, p. p. 45
#ICC = Intraclass correlations, proposion of the total variance in outcome that is attributable to the classes
getICC <-function(model){
  vc.model <- VarCorr(model)
  sigma <-model$sigma
  tau <- as.numeric(vc.model[2,2])
  icc <- (tau^2)/(tau^2 + sigma^2)
  return(icc)
}

#correlation helper to calculate confidence intervals of a correlation with bootstrapping
corrs <- function(df,name1, name2, indices){
  if(missing(indices)){
    return(cor(df[,c(name1,name2)],method="spearman",use="complete.obs")[name1,name2])
  }
  else {
    return(cor(df[indices,c(name1,name2)],method="spearman",use="complete.obs")[name1,name2])
  }
}

#helper function for the bootstrapping, calculates agreement on subsample
agrees <- function(df,name1,name2,indices){
  return(agree(df[indices,c(name1,name2)])$value)
}

#spearman correlations of intra-coder ratings (naive vs informed) and intercoder rating (CX informed vs C1 informed) and bootstrapped CIs
correlations <- function(df,coderID){
  intracor <- corrs(df[which(df$coderID==coderID),],"CXnoscale","CXscale")
  intracor.bs <- boot(data=df[which(df$coderID==coderID),c("CXnoscale","CXscale")], statistic=corrs, R=10000, name1="CXnoscale", name2="CXscale")
  intracor.cis <- boot.ci(intracor.bs, type="bca")
  intracor.cis <- intracor.cis$bca[1,c(4,5)]
  intercor <- corrs(df[which(df$coderID==coderID),],"C1scale","CXscale")
  intercor.bs <- boot(data=df[which(df$coderID==coderID),c("C1scale","CXscale")], statistic=corrs, R=10000, name1="C1scale",name2="CXscale")
  intercor.cis <- boot.ci(intercor.bs, type="bca")
  intercor.cis <- intercor.cis$bca[1,c(4,5)]
  intra <- c(intracor, intracor.cis)
  inter <- c(intercor, intercor.cis)
  info <- c("estimate","BS_lower-CI","BS_upper-CI")
  df.corrs <- data.frame(info,intra,inter)
  return(df.corrs)
}

#agreement with bootstrapped confidence intervals
agg_class_levels <- function(C1level,C3level,comment){
  cat(comment)
  percagg_classification <- agree(df.class_final[,c(C1level,C3level)])
  print(percagg_classification)
  fic.bs <- boot(data=df.class_final[,c(C1level,C3level)], statistic=agrees, R=10000, name1=C1level, name2=C3level)
  fic.cis <- boot.ci(fic.bs, type="bca")
  print(fic.cis)
}

#cohen's kappa with confidence intervals
irr_class_levels <- function(C1level,C3level,comment){
  cat(comment)
  c_kappa_classificationL <- kappa2(df.class_final[,c(C1level,C3level)], weight = "unweighted")
  c_kappa2_classificationL <- cohen.kappa(df.class_final[,c(C1level,C3level)], w=NULL, alpha=.05)
  print(c_kappa_classificationL)
  print(c_kappa2_classificationL$confid[1,])
  return(c_kappa_classificationL$value)
}

#calculating d' specificity
d_prime <- function(hit,fa,miss,cr){
  z_hit <- qnorm((hit/(hit+miss)),0,1)
  z_fa <- qnorm((fa/(fa+cr)),0,1)
  spec <- z_hit - z_fa
  return(spec)
}
######################### End functions


cat("################################# ANALYSES #################################\n")
#connecting to the ehealth4mdd database
u <- 'root'
p <- 'insy'
n <- 'ehealth4mdd'
n2 <- 'ehealth4mdd_CYPSY23'
h <- 'localhost'
ehealthdb = dbConnect(MySQL(), user=u, password=p, dbname=n, host=h)
ehealthdbCYPSY = dbConnect(MySQL(), user=u, password=p, dbname=n2, host=h)

###Database Queries
#query to expand on the interrater reliability analyses
qfuncs_ehealthdb <- 'select * from Function;'
query_funcs1 <- dbSendQuery(ehealthdb,qfuncs_ehealthdb)
query_funcs2 <- dbSendQuery(ehealthdbCYPSY,qfuncs_ehealthdb)
df.funcsehealthdb <- fetch(query_funcs1,n=-1)
df.funcsCYPSY <- fetch(query_funcs2,n=-1)

#query to describe current state of e-mental health systems for depression
q_g1 <- 'select s.SystemName, v.SystemKey, vf.VersionKey, vf.FunctionKey, v.BuildYear, v.Purpose, v.SupportType, v.Comorbidity, v.Duration, v.InterventionModules, v.TherapyType, v.Therapy, vf.Format, vf.eHDTS, f.FunctionSubSubType, f.FunctionSubType, f.FunctionType, f.Functionality, f.Therapy
from System as s, VersionFunction as vf, Function as f, Version as v 
where s.SystemKey=v.SystemKey AND v.VersionKey=vf.VersionKey AND vf.FunctionKey=f.FunctionKey;'
query_g1 <- dbSendQuery(ehealthdb, q_g1)
df.g1 <- fetch(query_g1,n=-1)
df.g1$BuildYear <- as.numeric(as.character(df.g1$BuildYear))
colnames(df.g1)[12] <- "AuthorTherapy"

#query to access the evaluation information per version in the database
q_g2 <- 'select v.SystemKey, v.VersionKey, e.EvaluationKey, e.Quality, e.MeasurementTPs, e.Control, e.Result, a.ArmKey, a.Npre, a.Npost 
from Evaluation as e, Version as v, VersionEvaluation as ve, Arm as a
where ve.VersionKey=v.VersionKey AND ve.EvaluationKey=e.EvaluationKey AND a.EvaluationKey=e.EvaluationKey;'
query_g2 <- dbSendQuery(ehealthdb, q_g2)
df.g2 <- unique(fetch(query_g2,n=-1))
df.g2$Quality2 <- df.g2$Quality
df.g2$Quality <- revalue(as.factor(df.g2$Quality), c("3"="1","2"="1"))

#query to access the measure information per evaluation and version in the database
q_g3 <- 'select v.SystemKey,ve.VersionKey, ve.EvaluationKey, em.MeasureKey, m.MeasureName, m.MeasureType 
from Version as v, VersionEvaluation as ve, EvaluationMeasure as em, Measure as m
where v.VersionKey=ve.VersionKey AND ve.EvaluationKey=em.EvaluationKey AND em.MeasureKey=m.MeasureKey;'
query_g3 <- dbSendQuery(ehealthdb, q_g3)
df.g3 <- fetch(query_g3,n=-1)

cat("#######LITERATURE IDENTIFICATION AND CODING\n")
cat("For these analyses please refer to [11]. The corresponding R scripts are stored in an open access, national database, 
    under https://doi.org/10.4121/uuid:7e7e91ab-7afc-4b48-8915-e2bc80b23c99 \n")

cat("###Identification\n")

#reading in the data
df.litscreen <- read.table("input/literature_screening_C1C2.txt", header=TRUE, sep="\t", fill=TRUE)

cat("##title testing\n")
df.title <- df.litscreen[which(df.litscreen$Stage=="title"),c("C1","C2")]
cat("Population: \n")
print(4266)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on title alone\n")
agree_tite <- agree(df.title)
print(agree_tite)
cat("\n")
# cat("#bootstrapped confidence intervals for percent agreement on title\n")
# title.bs <- boot(data=df.title, statistic=agrees, R=10000, name1="C1", name2="C2")
# title.cis <- boot.ci(title.bs, type="bca")
# print(title.cis)
# cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on title\n")
kappa_tite <- cohen.kappa(df.title, w=NULL, alpha=.05)
print(kappa_tite$kappa)
cat("\n")
# cat("confidence intervals for unweighted Cohen's kappa for agreement on title\n")
# print(kappa_tite$confid[1,])
# cat("\n")
cat("##abstract testing\n")
df.abstract <- df.litscreen[which(df.litscreen$Stage=="abstract"),c("C1","C2")]
cat("Population:\n")
print(1071)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on abstract\n")
agree_abte <- agree(df.abstract)
print(agree_abte)
cat("\n")
# cat("#bootstrapped confidence intervals for percent agreement on abstract\n")
# abstract.bs <- boot(data=df.abstract, statistic=agrees, R=10000, name1="C1", name2="C2")
# abstract.cis <- boot.ci(abstract.bs, type="bca")
# print(abstract.cis)
# cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on abstract\n")
kappa_abte <- cohen.kappa(df.abstract, w=NULL, alpha=.05)
print(kappa_abte$kappa)
cat("\n")
# cat("#confidence intervals for unweighted Cohen's kappa for agreement on abstract\n")
# print(kappa_abte$confid[1,])
# cat("\n")
cat("##full-paper testing\n")
df.full <- df.litscreen[which(df.litscreen$Stage=="fullpaper"),c("C1","C2")]
cat("Population: \n")
print(541)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on full article\n")
agree_fute <- agree(df.full)
print(agree_fute)
cat("\n")
# cat("#bootstrapped confidence intervals for percent agreement of full-paper\n")
# full.bs <- boot(data=df.full, statistic=agrees, R=10000, name1="C1", name2="C2")
# full.cis <- boot.ci(full.bs, type="bca")
# print(full.cis)
# cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on full paper\n")
kappa_fute <- cohen.kappa(df.full, w=NULL, alpha=.05)
print(kappa_fute$kappa)
cat("\n")
# cat("#confidence intervals for unweighted Cohen's kappa for agreement on full paper\n")
# print(kappa_fute$confid[1,])
# cat("\n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")

cat("###Coding\n")
df.functions <- merge(df.funcsCYPSY[,c("FunctionKey", "Functionality", "Therapy")],df.funcsehealthdb[,c("FunctionKey","FunctionSubSubType","FunctionSubType","FunctionType")], by="FunctionKey", all.x = TRUE)

cat("##Function identification\n")
#reading in the data
df.identifying <- read.table("input/function_identification_C1C3.txt", header=TRUE, sep="\t", fill=TRUE)
#preprocessing the data
df.identifying <- df.identifying[-which(is.na(df.identifying$FunctionKey)),]
df.identifying_expand <- unique(merge(df.identifying[,"VersionKey"], df.functions[,"FunctionKey"],by=NULL))
names(df.identifying_expand) <- c("VersionKey","FunctionKey")
df.identifying2 <- merge(df.identifying,df.identifying_expand, by=c("VersionKey","FunctionKey"),all.y=TRUE)
df.identifying2[is.na(df.identifying2)] <- 0

#calculating the different counts for hit, miss, false alarm, correct rejection
df.detection_ind <- df.identifying2 %>% group_by(C1, C3) %>% dplyr::summarise(counts=n())
hit <- df.detection_ind$counts[4]
cr <- df.detection_ind$counts[1]
miss <- df.detection_ind$counts[3]
fa <- df.detection_ind$counts[2]
phi <- phi(c(hit,fa,miss,cr))
detect_ind <- d_prime(hit,fa, miss,cr)
cat("#phi:\n")
print(phi)
cat("#d-prime:\n")
print(detect_ind)
cat("\n")
cat("\n")

cat("## Function classification\n")
#reading in the data
df.classification <- read.table("input/function_classification_C1C3.txt", header=TRUE, sep="\t")
#preprocessing the data
df.classificationC1 <- merge(df.classification[,c("Order_Initial","C1")],df.functions,by.x="C1",by.y="FunctionKey",all.x=TRUE)
df.classificationC3 <- merge(df.classification[,c("Order_Initial","C3")],df.functions,by.x="C3",by.y="FunctionKey",all.x=TRUE)

df.classificationC1[,c("Functionality","Therapy")] <- NULL
df.classificationC3[,c("Functionality","Therapy")] <- NULL

names(df.classificationC1) <- c("C1.L0","ID","C1.L1","C1.L2","C1.L3")
names(df.classificationC3) <- c("C3.L0","ID","C3.L1","C3.L2","C3.L3")

df.class_final <- merge(df.classificationC1,df.classificationC3,by="ID")

cat("##analyzing agreement at the different levels (L1-L3 shown in Figure 2)\n")
agg_class_levels("C1.L0","C3.L0","#Level0 percent agreement and bootstrapped confidence intervals:\n")
cat("\n")
cat("\n")
agg_class_levels("C1.L1","C3.L1","#Level1 percent agreement and bootstrapped confidence intervals:\n")
cat("\n")
cat("\n")
agg_class_levels("C1.L2","C3.L2","#Level2 percent agreement and bootstrapped confidence intervals:\n")
cat("\n")
cat("\n")
agg_class_levels("C1.L3","C3.L3","#Level3 percent agreement and bootstrapped confidence intervals:\n")
cat("\n")
cat("\n")

cat("##analyzing interrater reliability at the different levels (L1-L3 shown in Figure 2):\n")
l0_kappa <- irr_class_levels("C1.L0","C3.L0", "#Level0 with confidence intervals:\n")
cat("\n")
cat("\n")
l1_kappa <- irr_class_levels("C1.L1","C3.L1", "#Level1 with confidence intervals:\n")
cat("\n")
cat("\n")
l2_kappa <- irr_class_levels("C1.L2","C3.L2", "#Level2 with confidence intervals:\n")
cat("\n")
cat("\n")
l3_kappa <- irr_class_levels("C1.L3","C3.L3", "#Level3 with confidence intervals:\n")
cat("\n")
cat("\n")

cat("##mean of kappas:\n")
print(mean(c(l0_kappa,l1_kappa,l2_kappa,l3_kappa)))
cat("\n")
cat("\n")

cat("## Reliability analysis of eHDTS Scale\n")

#read in the data
df.validation <- read.table("input/validation_C1CX.txt", header=TRUE, sep="\t")

cat("Population:\n")
print(2224)
cat("#percent agreement and cohen's kappa for automation degree assigned to all functions by C2\n")
df.val <- df.validation[which(df.validation$coderID==2 & !is.na(df.validation$CXscale)),c("CXscale","C1scale")]
perc_agg <- agree(df.val)
print(perc_agg)
cat("\n")
# cat("#bootstrapped confidence intervals for percent agreement of eHDTS-scoring\n")
# ehdtss.bs <- boot(data=df.val, statistic=agrees, R=10000, name1="CXscale", name2="C1scale")
# ehdtss.cis <- boot.ci(ehdtss.bs, type="bca")
# print(ehdtss.cis)
cat("\n")
cat("#weighted Cohen's Kappa and significance test:\n")
c_kappa <- kappa2(df.val, weight = "squared")
print(c_kappa)
cat("\n")
# cat("#weighted Cohen's Kappa with confidence intervals:\n")
# c_kappa2 <- cohen.kappa(df.val, alpha=.05)
# print(c_kappa2$confid[2,])
# cat("\n")
# cat("\n")

cat("## Validity analysis of eHDTS Scale\n")
cat("For these analyses please refer to [11]. The corresponding R scripts are stored in an open access, national database, 
    under https://doi.org/10.4121/uuid:7e7e91ab-7afc-4b48-8915-e2bc80b23c99 \n")
cat("\n")
cat("For the origin of these numbers, please refer to [11].\n")

##INTRA-CODER Correlations: each coder coded the items once when unfamiliar with the scale levels and once when familiar 
##with the scale levels. Intracoder correlations correlate these two scores (naive and informed). Confidence intervals are obtained by bootstrapping.
##INTER-CODER Correlation: the codes assigned by the informed second coders are correlated with the informed codes assigned by C1.
#intervention correlation C4 naive with informed (intra) and informed C4 with informed C1 (inter)
# cat("Population:\n")
# print(1344)
# cat("Sample:\n")
# print(nrow(df.validation[which(df.validation$coderID==4),]))
# cat("Coders:\n")
# print(2)
# cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c4 <- correlations(df.validation,4)
#cat("\n")

#planning correlation C5 naive with informed (intra) and informed C5 with informed C1 (inter)
# cat("Population:\n")
# print(29)
# cat("Sample:\n")
# print(nrow(df.validation[which(df.validation$coderID==5),]))
# cat("Coders:\n")
# print(2)
# cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c5 <- correlations(df.validation,5)
#cat("\n")

#execution correlation C6 naive with informed (intra) and informed C6 with informed C1 (inter)
# cat("Population:\n")
# print(445)
# cat("Sample:\n")
# print(nrow(df.validation[which(df.validation$coderID==6),]))
# cat("Coders:\n")
# print(2)
# cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c6 <- correlations(df.validation,6)
#cat("\n")

#monitoring correlation C7 naive with informed (intra) and informed C7 with informed C1 (inter)
# cat("Population:\n")
# print(140)
# cat("Sample:\n")
# print(nrow(df.validation[which(df.validation$coderID==7),]))
# cat("Coders:\n")
# print(2)
# cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c7 <- correlations(df.validation,7)
#cat("\n")

#social correlation C8 naive with informed (intra) and informed C8 with informed C1 (inter)
# cat("Population:\n")
# print(266)
# cat("Sample:\n")
# print(nrow(df.validation[which(df.validation$coderID==8),]))
# cat("Coders:\n")
# print(2)
# cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c8 <- correlations(df.validation,8)
#cat("\n")

cat("#all items mixed correlation of C2 naive versus informed scores (intra) and informed C2 with informed C1 (inter)\n")
cat("Population:\n")
print(2224)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==2 & (!is.na(df.validation$CXnoscale) & !is.na(df.validation$CXscale))),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
corrc1c2 <- correlations(df.validation,2)
print(corrc1c2)
cat("\n")

cat("##mean inter-coder correlation:\n")
print(mean(c(corrc1c4[1,3],corrc1c5[1,3],corrc1c6[1,3],corrc1c7[1,3],corrc1c8[1,3],corrc1c2[1,3])))

cat("##mean intra-coder correlation for concurrent validity:\n")
print(mean(c(corrc1c4[1,2],corrc1c5[1,2],corrc1c6[1,2],corrc1c7[1,2],corrc1c8[1,2],corrc1c2[1,2])))
cat("\n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")


cat("####### RESULTS\n")
cat("#####CHARACTERIZATION\n")
cat("####RQ: What is the content and composition of e-health systems for depression?\n")
cat("#Total Number of Systems:\n")
svs <- length(unique(df.g1$SystemKey))
print(svs)
cat("\n")
cat("\n")

cat("#Total Number of Versions:\n")
nvs <- length(unique(df.g1$VersionKey))
print(nvs)
cat("\n")
cat("\n")

cat("### TABLE 2 \n")
cat("##Technology\n")
cat("\n")
cat("\n")

cat("#Number of Versions\n")
df.vps <- df.g1[,c("SystemKey","VersionKey")] %>% group_by(SystemKey) %>% unique(.) %>% dplyr::summarise(., vcount=n())
boxplot(df.vps$vcount, main="Distribution over Versions per System")
cat("summary of distribution of number of versions per system:\n")
print(psych::describe(df.vps$vcount))
cat("\n")
cat("\n")

cat("#Technology\n")
techkeys <- list(c(0, 5, 6), 7, 8, 9, 1, 21, 2, 3, 4, 10, 11, 12, 15, 13, 14,16,c(17,19,20))
technologies <- c("CD/DVD", "computer", "www", "e-mail", "telephone", "IVR", "text message", "mobile", "app", "AR", "VR", "virtual agent", "social media","multimedia","sensors","undefined","offline")
nvwithtech <- summary_statistics(df.g1[,c("SystemKey","VersionKey","Format")],"Format",c("SystemKey","VersionKey"),"value",techkeys,technologies) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of versions that make use of a certain technology ('none' comprises paper-and-pencil, face-to-face, and postal):\n")
print(data.frame(nvwithtech)[order(-nvwithtech$perc),])
cat("\n")
cat("\n")

cat("#Support Type\n")
suppkeys <- list(0,1,2,3,4,5)
supporttypes <- c("autonomous","lay-person","administrative","professional","therapist","blended/adjunct")
nvwithsupport <- summary_statistics(df.g1[,c("SystemKey","VersionKey","SupportType")],"SupportType",c("SystemKey","VersionKey"),"value",suppkeys,supporttypes) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of versions that have a specific type of support:\n")
print(data.frame(nvwithsupport)[order(-nvwithsupport$perc),])
cat("\n")
cat("\n")

cat("#Number of Functions\n")
df.fps <- df.g1[,c("VersionKey","FunctionKey")] %>% group_by(VersionKey) %>% unique(.) %>% dplyr::summarise(., fcount=n())
boxplot(df.fps$fcount, main="Distribution over Functions across Systems")
cat("summary of distribution of number of functions per version:\n")
print(psych::describe(df.fps$fcount))
cat("\n")
cat("\n")

cat("#Function Type\n")
nvwithfunctionst <- summary_statistics(df.g1[,c("SystemKey","VersionKey","FunctionSubType")],"FunctionSubType",c("SystemKey","VersionKey"),"value") %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of versions that have a specific function type:\n")
print(data.frame(nvwithfunctionst)[order(-nvwithfunctionst$perc),])
cat("\n")
cat("\n")

cat('#Sophistication\n')
df.ehdts_pv <- df.g1[,c("VersionKey","eHDTS")] %>% group_by(VersionKey) %>% dplyr::summarise(tsmean=mean(eHDTS))
cat('all:\n')
print(psych::describe(df.ehdts_pv$tsmean))
cat("\n")
cat("#intervention\n")
print(ehdts_subdf("INT") %>% .[,"tsmean"] %>% as.matrix(.) %>% psych::describe(.))
cat("#execution support\n")
print(ehdts_subdf("EXEC") %>% .[,"tsmean"] %>% as.matrix(.) %>% psych::describe(.))
cat("#social support\n")
print(ehdts_subdf("SOC") %>% .[,"tsmean"] %>% as.matrix(.) %>% psych::describe(.))
cat("#monitoring\n")
print(ehdts_subdf("MONITOR") %>% .[,"tsmean"] %>% as.matrix(.) %>% psych::describe(.))
cat("#planning support\n")
print(ehdts_subdf("PLAN") %>% .[,"tsmean"] %>% as.matrix(.) %>% psych::describe(.))
cat("\n")
cat("\n")

cat("##Therapy")
cat("\n")
cat("\n")

cat("#Comorbidity\n")
comorbkeys <- list(0,1,2,3,4,5)
comorbtypes <- c("none","addiction","physical","anxiety","insomnia","non-specific")
nvwithcomorb <- summary_statistics(df.g1[,c("SystemKey","VersionKey","Comorbidity")],"Comorbidity",c("SystemKey","VersionKey"),"value",comorbkeys,comorbtypes) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of versions that target a certain comorbidity:\n")
print(data.frame(nvwithcomorb)[order(-nvwithcomorb$perc),])
cat("\n")
cat("\n")

cat("#Purpose\n")
purposekeys <- list(0,1,2)
purposetypes <- c("treat","prevent","after-care")
nvwithpurpose <- summary_statistics(df.g1[,c("SystemKey","VersionKey","Purpose")],"Purpose",c("SystemKey","VersionKey"),"value",purposekeys,purposetypes) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of versions that have a specific therapeutic purpose:\n")
print(data.frame(nvwithpurpose)[order(-nvwithpurpose$perc),])
cat("\n")
cat("\n")

cat("#Duration\n")
df.g1$Duration <- as.numeric(as.character(df.g1$Duration))
df.dpv <- df.g1[which(!is.na(df.g1$Duration)),c("VersionKey","Duration")] %>% group_by(VersionKey) %>% unique(.) %>% dplyr::summarise(., dmean=mean(Duration))
boxplot(df.dpv$dmean, main="Duration in Weeks across Systems")
print(psych::describe(df.dpv$dmean))
cat("\n")
cat("\n")

cat("#Number of Modules\n")
df.impv <- df.g1[,c("VersionKey","InterventionModules")] %>% group_by(VersionKey) %>% unique(.) %>% dplyr::summarise(., immean=mean(InterventionModules))
boxplot(df.impv$immean, main="Number of Intervention Modules across Systems")
print(psych::describe(df.impv$immean))
cat("\n")
cat("\n")

cat("#Therapy Class\n")
therapykeys <- list(c(1,11,7),c(2,5,23,21,20),c(3,6,22,12),c(4,9,17,19),10,13,14,c(24,16,15,25))
therapytypes <- c("CBT","COG","BEH","INDEP","PP","IP","PD","OTH")
nvwiththerapy <- summary_statistics(df.g1[which(df.g1$Therapy!=0),c("SystemKey","VersionKey","Therapy")],"Therapy",c("SystemKey","VersionKey"),"value",therapykeys,therapytypes) %>% .[["nvc"]]
relen_nvwiththerapy <- data.frame(nvwiththerapy[rep(1:nrow(nvwiththerapy), nvwiththerapy[["count"]]),]) %>% .[,"value"] %>% relen(.)
cat("#number of versions and percentage of versions that target a certain comorbidity\n")
print(data.frame(nvwiththerapy)[order(-nvwiththerapy$perc),])
cat("\n")
cat("\n")

cat("Evaluation\n")
cat("\n")
cat("\n")

cat("#Number of Studies\n")
df.epv <- df.g2[,c("VersionKey","EvaluationKey")] %>% group_by(VersionKey) %>% unique(.) %>% dplyr::summarise(., ecount=n())
boxplot(df.epv$ecount, main="Distribution over Evaluations per Version")
print(psych::describe(df.epv$ecount))
cat("\n")
cat("\n")

cat("#Quality\n")
evalqualitykeys <- list(0,1)
evalqualitytypes <- c("non-comparative","comparative")
nvwitheval <- summary_statistics(unique(df.g2[,c("SystemKey","VersionKey","Quality")]),"Quality",c("SystemKey","VersionKey"),"value",evalqualitykeys,evalqualitytypes) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of systems that have been evaluated at a certain level:\n")
print(data.frame(nvwitheval)[order(-nvwitheval$perc),])
cat("\n")
cat("\n")

cat("#Control Group Type\n")
cgkeys <- list(1,2,3)
cgtypes <- c("waitlist","TAU","attention control")
nvwithcg <- summary_statistics(unique(df.g2[which(df.g2$Control!=0),c("SystemKey","VersionKey","Control")]),"Control",c("SystemKey","VersionKey"),"value",cgkeys,cgtypes) %>% .[["nvc"]] %>% data.frame(.)
cat("percentage of systems that have been evaluated with a certain type of control group:\n")
print(data.frame(nvwithcg)[order(-nvwithcg$perc),])
cat("number of systems evaluated at least once in a controlled trial:\n")
nvscontrolled <- nrow(unique(df.g2[which(df.g2$Control!=0),c("SystemKey","VersionKey","Control")]))
print(nvscontrolled)
cat("percentage:\n")
print(nvscontrolled/nvs)
cat("\n")
cat("\n")

cat("#Measures\n")
measurekeys <- list(c(34,35,52,9),c(8,6,7,54,10),12,as.numeric(setdiff(unique(df.g3$MeasureKey),c(34,35,52,9,8,6,7,54,10,12,49,2,4,23,18,3,31,32,43,60))),c(18,2,23,3,31,32,4,43,49,60))
measures <- c("PHQ","BDI","CES-D","Other Depression/Depression-Related","Non-Depression")
nvwithmeasure <- summary_statistics(df.g3[,c("SystemKey","VersionKey","MeasureKey")],"MeasureKey",c("SystemKey","VersionKey"),"value",measurekeys,measures) %>% .[["nvc"]] %>% data.frame(.)
cat("number of systems and percentage of systems that have been used a certain outcome measure (top 5 depression measures in this set):\n")
print(data.frame(nvwithmeasure)[order(-nvwithmeasure$perc),])
cat("\n")
cat("\n")
cat("### end table 2 \n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")

cat("###Versions\n")
cat("percentage of systems with only one version:\n")
print(nrow(df.vps[which(df.vps$vcount==1),])/nrow(df.vps))
cat("n:\n")
print(nrow(df.vps[which(df.vps$vcount==1),]))
cat("N:\n")
print(nrow(df.vps))
cat("\n")
cat("\n")

cat("number of systems with at least 5 versions:\n")
print(nrow(df.vps[which(df.vps$vcount>=5),]))
cat("\n")
cat("\n")

cat("ten systems with most versions:\n")
df.vps <- unique(merge(df.vps, df.g1[,c("SystemKey","SystemName")],by="SystemKey",all.x=TRUE))
print(head(df.vps[order(-df.vps$vcount),]))
cat("\n")
cat("\n")

cat("### Size and Functionality\n")
cat("How many versions offer intervention/support functions?\n")
nvwithfunction <- summary_statistics(df.g1[,c("SystemKey","VersionKey","FunctionType")],"FunctionType",c("SystemKey","VersionKey"),"value") %>% .[["nvc"]] %>% data.frame(.)
print(nvwithfunction)
cat("\n")
cat("\n")

cat("#Do versions have significantly more intervention than support functions?\n")
df.nfuncis <- df.g1[,c("VersionKey","FunctionType")] %>% group_by(VersionKey,FunctionType) %>% dplyr:::summarise(count=n())
df.nfuncis <- dcast(df.nfuncis,VersionKey~FunctionType, value.var="count")
cat("how many more support than intervention functions do versions have on average:\n")
df.nfuncis$ratio <- df.nfuncis$INT/df.nfuncis$SUPP
print(psych::describe(df.nfuncis$ratio))
cat("Wilcoxon test comparing number of intervention to the number of support functions:\n")
wlcxtest_nfuncis <- wilcox.test(df.nfuncis$INT,df.nfuncis$SUPP,paired=TRUE,conf.int=TRUE)
print(wlcxtest_nfuncis)
cat("Effect Size:\n")
print(wlcxtest_nfuncis$estimate/sqrt(2*length(df.nfuncis$VersionKey)))
cat("\n")
cat("\n")

cat("#Is the lack of guidance in autonomous systems compensated by additional functionality?\n")
cat("a: in general (direct social support excluded)\n")
df.nfuncpguid <- df.g1[which(df.g1$SupportType!=5 & df.g1$FunctionSubSubType %nin% c("Direct - Prof","Direct - NonProf")),c("VersionKey","SupportType","FunctionType")]
df.nfuncpguid$SupportType <- ifelse(df.nfuncpguid$SupportType==0,0,1)
df.nfuncpguid <- df.nfuncpguid %>% group_by(VersionKey) %>% mutate(count=n()) 
df.nfuncpguid$FunctionType <- NULL
df.nfuncpguid <- unique(df.nfuncpguid)
cat("linear regression model predicting the support type from the number of version functions:\n")
model.nfuncpguid <- lm(count ~ SupportType, data=df.nfuncpguid)
print(summary(model.nfuncpguid))
cat("\n")
cat("how do the number of functions in guided vs unguided systems compare?\n")
df.mfuncpguid <- df.nfuncpguid %>% group_by(SupportType) %>% dplyr::summarise(mfunc=mean(count))
print(df.mfuncpguid[1,2]/df.mfuncpguid[2,2])
cat("\n")
cat("b: intervention functions\n")
df.nifuncpguid <- df.g1[which((df.g1$SupportType!=5) & (df.g1$FunctionType %in% "INT")),c("VersionKey","SupportType","FunctionType")]
df.nifuncpguid$SupportType <- ifelse(df.nifuncpguid$SupportType==0,0,1)
df.nifuncpguid <- df.nifuncpguid %>% group_by(VersionKey,SupportType) %>% mutate(count=n())
df.nifuncpguid$FunctionType <- NULL
df.nifuncpguid <- unique(df.nifuncpguid)
boxplot(df.nifuncpguid$count, main="Number of intervention functions")
cat("mann-whitney u test for comparing number of intervention functions per support type:\n")
wlcxtest_nifuncpguid <- wilcox.test(df.nifuncpguid$count~as.factor(df.nifuncpguid$SupportType),paired=FALSE) #for U
print(wlcxtest_nifuncpguid)
wlcxtest_nifuncpguid <- wilcox_test(df.nifuncpguid$count~as.factor(df.nifuncpguid$SupportType),distribution="exact")
cat("Effect Size:\n")
print(wlcxtest_nifuncpguid@statistic@teststatistic/sqrt(length(df.nifuncpguid$count)))
cat("\n")
cat("c: support functions (direct social support excluded)\n")
df.nsfuncpguid <- df.g1[which(df.g1$SupportType!=5 & df.g1$FunctionType %nin% "INT" & df.g1$FunctionSubSubType %nin% c("Direct - Prof","Direct - NonProf")),c("VersionKey","SupportType","FunctionType")]
df.nsfuncpguid$SupportType <- ifelse(df.nsfuncpguid$SupportType==0,0,1)
df.nsfuncpguid <- df.nsfuncpguid %>% group_by(VersionKey) %>% mutate(count=n()) 
df.nsfuncpguid$FunctionType <- NULL
df.nsfuncpguid <- unique(df.nsfuncpguid)
boxplot(df.nsfuncpguid$count, main="Number of support functions")
cat("mann-whitney u test for comparing number of support functions per support type:\n")
wlcxtest_nsfuncpguid <- wilcox.test(df.nsfuncpguid$count~as.factor(df.nsfuncpguid$SupportType),paired=FALSE) #for U
print(wlcxtest_nsfuncpguid)
wlcxtest_nsfuncpguid <- wilcox_test(df.nsfuncpguid$count~as.factor(df.nsfuncpguid$SupportType),distribution="exact")
cat("Effect Size:\n") #check where this formula comes from!
print(wlcxtest_nsfuncpguid@statistic@teststatistic/sqrt(length(df.nsfuncpguid$count)))
cat("\n")
cat("\n")

cat("### Therapeutic Aspects\n")
cat("#Therapy Class with all CBT-related versus other therapies\n")
therapykeys <- list(c(1,11,7,2,5,23,21,20,3,6,22,12),c(4,9,17,19,10,13,14,24,16,15,25))
therapytypes <- c("CBT-Related","NOT-CBT")
nvwiththerapy <- summary_statistics(df.g1[which(df.g1$Therapy!=0),c("SystemKey","VersionKey","Therapy")],"Therapy",c("SystemKey","VersionKey"),"value",therapykeys,therapytypes) %>% .[["nvc"]]
relen_nvwiththerapy <- data.frame(nvwiththerapy[rep(1:nrow(nvwiththerapy), nvwiththerapy[["count"]]),]) %>% .[,"value"] %>% relen(.)
cat("#number of versions and percentage of versions that target a certain comorbidity\n")
print(data.frame(nvwiththerapy)[order(-nvwiththerapy$perc),])
cat("\n")
cat("\n")

cat("#how are the different therapies represented depending on the treatment purpose?\n")
df.tpp <- split_list(df.g1[which(df.g1$Purpose != 2),c("VersionKey","TherapyType","Purpose")],"TherapyType",c("VersionKey","Purpose")) %>% unique(.) 
model.tpp <- glm(as.numeric(as.character(Purpose)) ~ value, family=binomial(link='logit'), data=df.tpp)
print(summary(model.tpp))
cat("Odds Ratio:\n")
print(exp(model.tpp$coefficients))
cat("CIs:\n")
print(exp(confint(model.tpp)))
cat("ANOVA:\n")
print(Anova(model.tpp, type='II', test="Wald"))
cat("Pseudo-R-squared:\n")
print(nagelkerke(model.tpp))
cat("\n")
cat("\n")

cat("#how is guidance represented according to version purpose?\n")
df.guidpp <- df.g1[which(df.g1$Purpose != 2),c("VersionKey","SupportType","Purpose")] %>% unique(.)
df.guidpp$Purpose <- as.numeric(df.guidpp$Purpose)
df.guidpp$SupportType <- ifelse(df.guidpp$SupportType==0,2,1) #making guidance the reference class
model.guidpp <- glm(as.numeric(as.character(Purpose)) ~ SupportType, family=binomial(link='logit'), data=df.guidpp)
print(summary(model.guidpp))
cat("Odds Ratio:\n")
print(exp(model.guidpp$coefficients))
cat("CIs:\n")
print(exp(confint(model.guidpp)))
cat("ANOVA:\n")
print(Anova(model.guidpp, type='II', test="Wald"))
cat("Pseudo-R-squared:\n")
print(nagelkerke(model.guidpp))
cat("\n")
cat("\n")

cat("### Evaluation\n")
cat("percentage and number of versions that were evaluated only once with end users:\n")
oneepv <- nrow(df.epv[which(df.epv$ecount==1),])
print(oneepv/nrow(df.epv))
print(oneepv)
cat("number of depression measures:\n")
print(length(unique(df.g3[which(df.g3$MeasureKey %in% as.numeric(setdiff(unique(df.g3$MeasureKey),c(49,2,4,23,18,3,31,32,43,60,5,26,33,36,41,44,50,51,53,55,61)))),"MeasureType"])))
cat("number of depression-related measures:\n")
print(length(unique(df.g3[which(df.g3$MeasureType=="deprel"),"MeasureName"])))
cat("\n")
cat("\n")

cat("#### end RQ1\n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")

cat("##### TECHNOLOGICAL SOPHISTICATION\n")
cat("#### RQ2: What is the technological state-of-the-art of e-health systems for depression?\n")
cat("\n")
cat("\n")

cat('### Systems\n')

cat('#Ranking of systems according to their average degree of technological sophistication\n')
df.ehdtsranking <- df.g1[,c("SystemKey","VersionKey","eHDTS")] %>% group_by(SystemKey,VersionKey) %>% dplyr::summarise(., fcount=n(), tssum=sum(eHDTS), tsmean=mean(eHDTS), tsweighted=tsmean*((fcount-1)/(21-1))) %>% group_by(SystemKey) %>% dplyr::summarise(., mfcount=mean(fcount), mtssum=mean(tssum), mtsmean=mean(tsmean), mtsweight=mean(tsweighted)) %>% data.frame(.)
df.ehdtsranking2 <- merge(df.ehdtsranking, unique(df.g1[,c("SystemKey","SystemName","AuthorTherapy")]), by="SystemKey",all.x=TRUE)
cat("a: top ten systems sorted by average technological sophistication:\n")
print(head(df.ehdtsranking2[order(-df.ehdtsranking2$mtsmean),c("SystemName","AuthorTherapy","mfcount","mtsmean")], n=10))
cat("\n")
cat("b: top ten systems sorted by average technological sophistication:\n")
print(head(df.ehdtsranking2[order(-df.ehdtsranking2$mtsweight),c("SystemName","AuthorTherapy","mfcount","mtsmean","mtsweight")], n=10))
cat("\n")
cat("c: percentage of systems scoring at or above 2:\n")
ecdffunc <- ecdf(df.ehdtsranking$mtsmean)
ecdffunc2 <- ecdf(df.ehdtsranking$mtsweight)
print(1-ecdffunc(2))
cat("n:\n")
print(133*(1-ecdffunc(2)))
cat("\n")
cat("d: distribution over therapies for higher tech systems:\n")
df.ehdtsranking3 <- merge(df.ehdtsranking2, unique(df.g1[,c("SystemKey","TherapyType")]),by="SystemKey",all.x=TRUE)
therapydisthightech <- df.ehdtsranking3[which(df.ehdtsranking3$mtsmean>2),] %>% group_by(TherapyType) %>% dplyr::summarise(count=n())
print(data.frame(therapydisthightech))
cat("\n")

cat("#Does the size of the system correlate with avrg eHDTS score of a system?\n")
df.corrsizewehdts <- df.g1[,c("VersionKey","eHDTS")] %>% group_by(VersionKey) %>% dplyr::summarise(., fcount=n(),mehdts=mean(eHDTS))
corrsizewehdts <- rcorr(df.corrsizewehdts$fcount,df.corrsizewehdts$mehdts)
cat("Correlation between # of functions and avrg ehdts:\n")
print(corrsizewehdts)
cat("degrees of freedom:\n")
print((corrsizewehdts$n[1,1])-2)
cat("\n")
cat("\n")

cat("#Is the eHDTS a good predictor of system purpose?\n")
df.ehdtspurpose <- df.g1[which(df.g1$Purpose!=2),c("VersionKey","eHDTS","Purpose")] %>% group_by(VersionKey,Purpose) %>% dplyr::summarise(mehdts=mean(eHDTS))
model.ehdtspurpose <- glm(as.numeric(as.character(Purpose))~mehdts, family=binomial(link='logit'), data=df.ehdtspurpose)
cat("Logistic regression model predicting system purpose from eHDTS:\n")
print(summary(model.ehdtspurpose))
cat("Odds Ratio:\n")
print(exp(model.ehdtspurpose$coefficients))
cat("CIs:\n")
print(exp(confint(model.ehdtspurpose)))
cat("ANOVA:\n")
print(Anova(model.ehdtspurpose, type='II', test="Wald"))
cat("Pseudo-R-squared:\n")
print(nagelkerke(model.ehdtspurpose))
cat("\n")
cat("\n")

cat("#Is the lack of guidance in autonomous systems compensated by more technological sophistication (direct social support excluded)?\n")
df.ehdtspguid <- df.g1[which(df.g1$SupportType!=5),c("VersionKey","SupportType","eHDTS")]
df.ehdtspguid$SupportType <- ifelse(df.ehdtspguid$SupportType==0,0,1)
df.ehdtspguid <- df.ehdtspguid %>% group_by(VersionKey,SupportType) %>% dplyr::summarise(mehdts=mean(eHDTS)) 
cat("logistic regression model predicting the support type from eHDTS:\n")
model.ehdtspguid<- glm(SupportType ~ mehdts, family=binomial(link='logit'), data=df.ehdtspguid)
print(summary(model.ehdtspguid))
cat("Odds Ratio:\n")
print(exp(model.ehdtspguid$coefficients))
cat("CIs:\n")
print(exp(confint(model.ehdtspguid)))
cat("ANOVA:\n")
print(Anova(model.ehdtspguid, type='II', test="Wald"))
cat("Pseudo-R-squared:\n")
print(nagelkerke(model.ehdtspguid))
cat("\n")
cat("\n")

cat("#Is the eHDTS a good predictor of evaluation quality?\n")
df.ehdtsequal <- merge(df.corrsizewehdts,unique(df.g2[,c("VersionKey","Quality")]),by="VersionKey",all=TRUE)
df.ehdtsequal2 <- merge(df.corrsizewehdts,unique(df.g2[,c("VersionKey","Quality2")]), by="VersionKey",all=TRUE)
model.ehdtsequal <- glm(as.numeric(as.character(Quality))~mehdts, family=binomial(link='logit'), data=df.ehdtsequal)
cat("Logistic regression model predicting evaluation quality from eHDTS:\n")
print(summary(model.ehdtsequal))
cat("Odds Ratio:\n")
print(exp(model.ehdtsequal$coefficients))
cat("CIs:\n")
print(exp(confint(model.ehdtsequal)))
cat("ANOVA:\n")
print(Anova(model.ehdtsequal, type='II', test="Wald"))
cat("Pseudo-R-squared:\n")
print(nagelkerke(model.ehdtsequal))
cat("\n")
cat("\n")

cat("How many RCTs evaluate systems that score at or below 2 on average on the eHDTS scales?\n")
df.rctqualehdts <- df.ehdtsequal2[which(df.ehdtsequal2$Quality==3),] %>% mutate(bin = cut(mehdts, breaks=seq(from=0, to=4, by=1), right=FALSE))
df.rctqualehdts <- df.rctqualehdts %>% group_by(bin) %>% dplyr::summarise(count=n(), perc=count/nrow(df.rctqualehdts))
df.rctqualehdts <- data.frame(df.rctqualehdts)
cat("number of RCTs evaluating systems scoring below 2:\n")
rctsbelow2 <- df.rctqualehdts[1,2]+df.rctqualehdts[2,2]
print(rctsbelow2)
cat("\n")
cat("number of RCTs evaluating systems of any score:\n")
rctcount <- sum(df.rctqualehdts$count)
print(rctcount)
cat("\n")
cat("percentage of RCTs evaluating systems with a score below 2:\n")
print(rctsbelow2/rctcount)
cat("\n")
cat("#How many RCTs were conducted per eHDTS interval from [0,1), [1,2), [2,3), [3,4)?\n")
print(df.rctqualehdts)
cat("\n")
cat("\n")

cat("### Functions\n")

cat("#Do different types of functions differ in their technological sophistication?\n")
cat("support functions:\n")
print(psych::describe(df.g1[which(df.g1$FunctionType=="SUPP"),"eHDTS"]))
cat("intervention functions:\n")
print(psych::describe(df.g1[which(df.g1$FunctionType=="INT"),"eHDTS"]))
cat("\n")
cat("\n")

cat("#Do intervention and support functions differ in their technological sophistication?\n")
df.tspft <- df.g1[,c("VersionKey","FunctionType","eHDTS")]
df.tspft$FunctionType <- ifelse(df.tspft$FunctionType=="INT",0,1)
model0.tspft <- lme(eHDTS ~ 1, data=df.tspft,random=~1|VersionKey, method="ML")
model1.tspft <- lme(eHDTS ~ FunctionType, data=df.tspft,random=~1|VersionKey, method="ML")
cat("model0 having the mean ehdts per version as predictor, regardless of function type:\n")
print(summary(model0.tspft))
cat("model1 having the function type as fixed effect predictor and versionkey as random effect predictor:\n")
print(summary(model1.tspft))
cat("F-values:\n")
print(anova(model1.tspft))
cat("R squared:\n")
getRSquaredLevel1(model0.tspft,model1.tspft)
cat("\n")
cat("\n")

cat("#Do the different support functions significantly differ in their technological sophistication?\n")
df.tspsft <- df.g1[which(df.g1$FunctionType!="INT"),c("VersionKey","FunctionSubType","eHDTS")]
model0.tspsft <- lme(eHDTS ~ 1, data=df.tspsft,random=~1|VersionKey, method="ML")
model1.tspsft <- lme(eHDTS ~ FunctionSubType, data=df.tspsft,random=~1|VersionKey, method="ML")
cat("model0 having the mean ehdts per version as predictor, regardless of function subtype:\n")
print(summary(model0.tspsft))
cat("model1 having the function subtype as fixed effect predictor and versionkey as random effect predictor:\n")
print(summary(model1.tspsft))
cat("analysis of variance of variance of the model to determine whether there are differences between the support subtypes:\n")
print(anova(model1.tspsft))
cat("R squared:\n")
getRSquaredLevel1(model0.tspsft,model1.tspsft)
cat("\n")
cat("\n")

cat("#How automated are certain subsubfunctions per underlying therapy (therapy as claimed by the authors)?\n")
df.heatmaps <- split_list(df.g1[which(df.g1$eHDTS != 0),c("SystemKey","VersionKey","FunctionSubSubType","FunctionSubType","TherapyType","eHDTS")],"TherapyType",c("SystemKey","VersionKey","FunctionSubSubType","FunctionSubType","eHDTS"))
names(df.heatmaps)[names(df.heatmaps) == 'value'] <- 'TherapyType'
df.heatmaps$FunctionSubType <- revalue(df.heatmaps$FunctionSubType,c("SOC"="SS","PLAN"="SP","MONITOR"="SM","EXEC"="SE"))
df.heatmaps$FunctionSubType <- paste("(", df.heatmaps$FunctionSubType,sep="");
df.heatmaps <- df.heatmaps %>% unite(Function, FunctionSubType, FunctionSubSubType,sep=") ",remove=FALSE)
df.heatmaps_summary1 <- df.heatmaps[,c("VersionKey","Function")] %>% group_by(Function) %>% unique(.) %>% dplyr::summarise(TherapyType="Total",nvers=n())
df.heatmaps_summary4 <- df.heatmaps[,c("VersionKey","TherapyType")] %>% group_by(TherapyType) %>% unique(.) %>% dplyr::summarise(Function="Total",nvers=n())
df.heatmaps_freq2 <- df.heatmaps[,c("VersionKey","Function","TherapyType")] %>% group_by(TherapyType,Function) %>% unique(.) %>% dplyr::summarise(., nvers=n())
df.heatmaps_summary <- df.heatmaps[,c("VersionKey","Function","TherapyType","eHDTS")] %>% group_by(TherapyType,Function) %>% dplyr::summarise(.,ehdtsmean=mean(eHDTS,na.rm=TRUE),ehdtsmax=max(eHDTS)) %>% merge(df.heatmaps_freq2, ., by=c("Function","TherapyType"))
df.heatmaps_summary2 <- df.heatmaps_summary %>% group_by(Function) %>% dplyr::summarise(TherapyType="Total", ehdtsmean=mean(ehdtsmean),ehdtsmax=max(ehdtsmax))
df.heatmaps_summary3 <- df.heatmaps_summary %>% group_by(TherapyType) %>% dplyr::summarise(Function="Total", ehdtsmean=mean(ehdtsmean),ehdtsmax=max(ehdtsmax))
df.heatmaps_summary12 <- merge(df.heatmaps_summary1,df.heatmaps_summary2,by=c("Function","TherapyType"))
df.heatmaps_summary34 <- merge(df.heatmaps_summary3,df.heatmaps_summary4,by=c("Function","TherapyType"))
df.heatmaps_summary <- rbind.fill(rbind.fill(df.heatmaps_summary,df.heatmaps_summary12),df.heatmaps_summary34)

#Check-heatmaps: Are there significant differences over time in the number of functions and the automation degree -> to check for time influence in heatmaps?
df.byttehdts <- split_list(df.g1[which(df.g1$eHDTS!=0),c("BuildYear","FunctionSubType","FunctionSubSubType","TherapyType","eHDTS")],"TherapyType",c("BuildYear","FunctionSubType","FunctionSubSubType","eHDTS")) %>%  unite(Function, FunctionSubType, FunctionSubSubType,sep="_",remove=FALSE)
df.byttehdts <- df.byttehdts %>% group_by(BuildYear,Function,value) %>% mutate(nfuncs=n(),mehdts=mean(eHDTS)) %>% .[,c("BuildYear","value","nfuncs","mehdts")] %>% unique(.)
#are there significant differences over time in the number of functions per therapy type?
model.byftehdts <- lm(nfuncs ~ BuildYear + value + BuildYear*value, data=unique(df.byttehdts))
#are there significant differences over time in the average ehdts score per therapy type?
model.byttehdts <- lm(mehdts ~ BuildYear + value + BuildYear*value, data=unique(df.byttehdts))

cat("Is there a relationship between heatmaps?\n")
cat("a: between the implementation frequency and average sophistication:\n")
corrfs <- rcorr(df.heatmaps_summary$nvers,df.heatmaps_summary$ehdtsmean, type='pearson')
print(corrfs)
cat("degrees of freedom:\n")
print(corrfs$n[1,1]-2)
cat("\n")
cat("b: between the implementation frequency and maximum sophistication:\n")
corrfm <- rcorr(df.heatmaps_summary$nvers,df.heatmaps_summary$ehdtsmax, type='pearson')
print(corrfm)
cat("degrees of freedom:\n")
corrfm$n[1,1]-2
cat("\n")
#c: between the average sophistication and maximum sophistication:
corrsm <- rcorr(df.heatmaps_summary$ehdtsmean,df.heatmaps_summary$ehdtsmax, type='pearson')
#degrees of freedom:
#corrsm$n[1,1]-2
cat("\n")
cat("\n")

cat("#### end RQ2\n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")

cat("#### Developments over Time\n")
cat("#RQ3: How have systems for depression evolved over the years?\n")
cat("\n")
cat("\n")

cat("#Comparing number of systems between 2000 and 2014:\n")
df.spy <- df.g1[,c("SystemKey","BuildYear")] %>% group_by(SystemKey) %>% dplyr::summarise(., bymin=min(BuildYear)) %>% .[,c("SystemKey","bymin")] %>% group_by(bymin) %>% dplyr::summarise(., systems=n())
cat("Number of systems developed in 2000: ")
nsystems2000 <- as.integer(df.spy[which(df.spy$bymin==2000),"systems"])
print(nsystems2000)
cat("\n")
cat("Number of systems developed in 2014: ")
nsystems2014 <- as.integer(df.spy[which(df.spy$bymin==2014),"systems"])
print(nsystems2014)
cat("\n")
cat("How many more systems were developed in 2014 compared to 2000: ")
print(nsystems2014/nsystems2000)
cat("\n")
cat("\n")

cat("#Do versions get bigger in terms of their offered functionality over time?\n")
df.fpy <- df.g1[,c("VersionKey","BuildYear")] %>% group_by(VersionKey,BuildYear) %>% dplyr::summarise(., fcount=n())
model.fpy <- lm(fcount ~ BuildYear, data=df.fpy)
print(summary(model.fpy))
cat("\n")
cat("\n")

cat("#Do versions get more technologically sophisticated over time?\n")
df.ehdtspy <- df.g1[, c("VersionKey","BuildYear","eHDTS")] %>% group_by(VersionKey,BuildYear) %>% dplyr::summarise(.,mehdts=mean(eHDTS,na.rm=TRUE))
model.ehdtspby <- lm(mehdts ~ BuildYear, data=df.ehdtspy[which(df.ehdtspy$mehdts!=0),])
print(summary(model.ehdtspby))
cat("\n")
cat("\n")

cat("#For the systems with more than 1 version, is the version number (according to BuildYear) a good predictor for size?\n")
df.vnpby <- df.g1[,c("SystemKey","VersionKey","BuildYear","eHDTS")] %>% group_by(SystemKey,VersionKey,BuildYear) %>% dplyr::summarise(nfuncs=n(),mehdts=mean(eHDTS)) %>% group_by(SystemKey) %>% mutate(nv=n(),vnum=dense_rank(BuildYear))
model0.vnpnfunc <- lme(nfuncs ~ 1, data=df.vnpby[which(df.vnpby$nv>1),], random=~1|SystemKey, method="ML")
model1.vnpnfunc <- lme(nfuncs ~ vnum, data=df.vnpby[which(df.vnpby$nv>1),], random=~1|SystemKey, method="ML")
r2vnpnfunc <- getRSquaredLevel1(model0.vnpnfunc,model1.vnpnfunc)
cat("results of the multi-level linear model with version number as predictor of version size, with SystemKey as the random effect, versions are nested within systems:\n")
print(summary(model1.vnpnfunc))
cat("F-values:\n")
print(anova(model1.vnpnfunc))
cat("R-squared:\n")
print(r2vnpnfunc)
cat("\n")
cat("\n")

cat("#For the systems with more than 1 version, is the version number (according to BuildYear) a good predictor for eHDTS?\n")
model0.vnpnehdts <- lme(mehdts ~ 1, data=df.vnpby[which(df.vnpby$nv>1),],random=~1|SystemKey, method="ML")
model1.vnpnehdts <- lme(mehdts ~ vnum, data=df.vnpby[which(df.vnpby$nv>1),],random=~1|SystemKey, method="ML")
r2vnpnehdts <- getRSquaredLevel1(model0.vnpnehdts,model1.vnpnehdts)
cat("results of the multi-level linear model with version number as predictor of version ehdts, with SystemKey as the random effect, versions are nested within systems:\n")
print(summary(model1.vnpnehdts))
cat("F-values:\n")
print(anova(model1.vnpnehdts))
cat("R-squared:\n")
print(r2vnpnehdts)
cat("\n")
cat("\n")

cat("#For the systems with more than 1 version, is the version number (according to BuildYear) a good predictor for evaluation quality?\n")
df.evalqpt <- merge(unique(df.g2[,c("VersionKey","Quality")]),df.vnpby[,c("SystemKey","VersionKey","nv","vnum")],by="VersionKey",all.x=TRUE)
df.evalqpt <- df.evalqpt[which(df.evalqpt$nv>1),]
df.evalqpt$Quality <- as.numeric(as.character(df.evalqpt$Quality))
model0.evalqpt <- lme(Quality ~ 1, data=df.evalqpt, random=~1|SystemKey, method="ML", na.action=na.omit)
model1.evalqpt <- lme(Quality ~ vnum, data=df.evalqpt, random=~1|SystemKey, method="ML", na.action=na.omit)
r2evalpt <- getRSquaredLevel1(model0.evalqpt,model1.evalqpt)
cat("results of the multi-level linear model with version number as predictor of evaluation quality, with SystemKey as the random effect, versions are nested within systems:\n")
summary(model1.evalqpt)
cat("F-values:\n")
print(anova(model1.evalqpt))
cat("R-squared:\n")
print(r2evalpt)
cat("\n")
cat("\n")

cat("#### end RQ3\n")
cat("\n")
cat("\n")
cat("\n")
cat("\n")

######################### End analyses

sink()


########################PLOTS

#Plot theme
jp1theme <- theme(
  plot.title=element_blank(),
  plot.margin=unit(c(10,5,5,5),"mm"),
  #plot.title = element_text(hjust = 0.5),
  legend.title = element_text(family="Times New Roman", size = 12),
  legend.text = element_text(family="Times New Roman", size = 12, margin=margin(0,0,0,5)),
  legend.spacing.y=unit(.2, "cm"),
  legend.key = element_rect(colour = NA), 
  axis.title.y=element_text(family="Times New Roman", size=14,face="bold", vjust=2),
  axis.title.x=element_text(family="Times New Roman", size=14, face="bold", vjust=-2),
  axis.text.y = element_text(family="Times New Roman", size=12),
  axis.text.x = element_text(family="Times New Roman", size=12, angle=45,vjust=.7))

#Figure 3: how many versions of prevention or treatment systems exist for a specific therapy type?
df.tpp_plot <- df.tpp %>% group_by(value,Purpose) %>% dplyr::summarise(.,count=n()) %>% rbind(.,list(value="PD",Purpose="1",count=0))
df.tpp_plot$Purpose <- revalue(df.tpp_plot$Purpose,c("0"="treat","1"="prevent"))
tpp_plot <- ggplot(df.tpp_plot, alpha = 0.2, aes(x = value, y=count, fill=Purpose))  + 
  geom_bar(stat="identity",position=position_dodge(width=.75), width=.75, color="black") +
  scale_fill_brewer(palette="Paired") +
  labs(x = "Therapy", y = "Number of versions") +
  jp1theme+
  theme(legend.direction="vertical",legend.position = c(.73,.88), legend.title=element_blank()) +
  ggtitle("Number of versions of a specific Therapy Type according to Treatment Purpose")
tpp_plot
dev.print(png, file = "output/figure3.png", width = 618, height = 464)

#Figure 4: how many autonomous/guided versions exist for prevention and for treatment?
# save figure as png with width=432, height=464
df.guidpp_plot <- df.guidpp %>% group_by(SupportType,Purpose) %>% dplyr::summarise(.,count=n())
df.guidpp_plot$Purpose <- ifelse(df.guidpp_plot$Purpose==0,"treat","prevent")
df.guidpp_plot$SupportType <- ifelse(df.guidpp_plot$SupportType==2,"autonomous","guided")
guidpp_plot <- ggplot(df.guidpp_plot, alpha=0.2, aes(x = Purpose, y = count, fill=SupportType))+
  geom_bar(stat="identity",position=position_dodge(width=.75), width=.75, color="black") +
  scale_fill_brewer(palette="Paired") +
  labs(x = "Purpose", y = "Number of versions") +
  expand_limits(y=0) +
  jp1theme+
  theme(legend.direction="vertical",legend.position = c(.03,.88), legend.title=element_blank(), axis.text.x = element_text(angle=0)) +
  ggtitle("Number of versions that are guided vs autonomous according to Treatment Purpose")
guidpp_plot
dev.print(png, file = "output/figure4.png", width = 432, height = 464)

#Figure 5: What is the cumulative density of systems across the levels of the ehdts scale?
df.ehdtsps <- df.ehdtsranking %>% mutate(., ecdf=ecdffunc(mtsmean), ecdf2=ecdffunc2(mtsweight))
df.therapy<- unique(split_list(df.g1[,c("SystemKey","VersionKey","TherapyType")],"TherapyType","SystemKey"))
df.cumulative <- merge(df.ehdtsps,df.therapy,by="SystemKey",all.y = TRUE)
df.ehdtspsbest <- df.cumulative %>% group_by(value) %>% summarise(.,maxes=max(mtsmean)) %>% mutate(.,ecdf=ecdffunc(maxes))

#for Multimedia Appendix 9, list of automation degree deciles of cumulative density
quants <- c(.1,.2,.3,.4,.5,.6,.7,.8,.9,1)
#unweighted deciles
unweighted_dec <- quantile(df.ehdtsranking$mtsmean, quants)
#weighted deciles
weighted_dec <- quantile(df.ehdtsranking$mtsweight, quants)
cat("#MULTIMEDIA APPENDIX 9: unweighted cumulative density deciles:\n")
print(unweighted_dec)
cat("\n")
cat("#MULTIMEDIA APPENDIX 9: weighted cumulative density deciles:\n")
print(unweighted_dec)

# save figure as png with width=724, height=485
cumdens <- ggplot(df.cumulative, aes(x = mtsmean, y=ecdf)) + 
  geom_line()+
  geom_point(data=df.ehdtspsbest, aes(x=maxes, y=ecdf), color="blue") +
  geom_text_repel(data=df.ehdtspsbest,
                  force=2,
                  aes(x=maxes, y=ecdf, label=value),
                  hjust=1.5,
                  nudge_y= 0.05,
                  direction= "x",
                  angle= 0,
                  vjust= 0,
                  segment.size = 0.2) +
  scale_y_continuous(name="Cumulative percentage of systems",breaks=c(0,.1,.2,.3,.4,.5,.6,.7,.8,.9,1), limits=c(0,1))+
  scale_x_continuous(breaks=c(0,1,2,3,4),limits=c(0,4.2),labels = c("0\noffline","1\ninformational","2\ndata entry","3\nform response","4\ncontent response")) +
  jp1theme +
  xlab("eHDTS scale value") +
  theme(axis.text.x = element_text(angle=0)) +
  scale_color_brewer(palette="Blues") +
  ggtitle("Cumulative Density Function of eHDTS of Systems") 
cumdens
dev.print(png, file = "output/figure5.png", width = 724, height = 485)

#Figure 6: How many versions are there with a specific support function given their underlying therapy?
# save all heatmaps as png with width=848, height=769
#df.heatmaps_summary$Labels <- gsub("^.*?)","",df.heatmaps_summary$Function)
df.heatmaps_summary$Labels <- revalue(df.heatmaps_summary$Function, c("(INT) PP" = "(INT) (PP) Positive Psychology",
                                       "(INT) PD" = "(INT) (PD) Psychodynamic",
                                       "(INT) BEH" = "(INT) (BEH) Behavioral",
                                       "(INT) COG" = "(INT) (COG) Cognitive",
                                       "(INT) IP" = "(INT) (IP) Interpersonal",
                                       "(INT) INDEP" = "(INT) (INDEP) Independent",
                                       "(INT) CBT" = "(INT) (CBT) Cognitive Behavioral",
                                       "(INT) OTH" = "(INT) (OTH) Other"))
heatmap_count <- ggplot(df.heatmaps_summary, aes(TherapyType, Labels)) + 
  geom_tile(aes(fill = nvers), color = "white") +
  #scale_y_discrete(limits=(df.heatmaps_summary$Labels)[order(df.heatmaps_summary$Function)])
  geom_text(aes(label=nvers)) +
  scale_fill_gradient(low = "white", high = "steelblue") +
  ylab("Functions") +
  xlab("Therapeutic intervention class") + 
  jp1theme +
  theme(legend.title=element_text(size = 10), 
        legend.position = "bottom", 
        legend.justification="center") + 
  labs(fill = "Number\nof versions")
heatmap_count
dev.print(png, file = "output/figure6.png", width = 848, height = 769)

#Figure 7: How automated on average are versions with a specific support function given their underlying therapy?
heatmap_avrg <- ggplot(df.heatmaps_summary, aes(TherapyType, Labels)) +
  geom_tile(aes(fill = ehdtsmean), color = "white") +
  geom_text(aes(label=round(ehdtsmean,2))) +
  scale_fill_gradient(low = "white", high = "steelblue") +
  ylab("Functions") +
  xlab("Therapeutic intervention class") + 
  jp1theme +
  theme(legend.title=element_text(size = 10), 
        legend.position = "bottom", 
        legend.justification="center") +
  labs(fill = "Mean eHDTS\nof functions")
heatmap_avrg
dev.print(png, file = "output/figure7.png", width = 848, height = 769)

#MMA10: How automated is the most automated version with a specific support function given their underlying therapy?
heatmap_max <- ggplot(df.heatmaps_summary, aes(TherapyType, Labels)) +
  geom_tile(aes(fill = ehdtsmax), color = "white") +
  geom_text(aes(label=round(ehdtsmax,2))) +
  scale_fill_gradient(low = "white", high = "steelblue") +
  ylab("Functions") +
  xlab("Therapeutic intervention class") +
  jp1theme +
  theme(legend.title=element_text(size = 10), 
        legend.position = "bottom", 
        legend.justification="center") +
  labs(fill = "Maximum eHDTS\nof functions")
heatmap_max
dev.print(png, file = "output/mma10.png", width = 848, height = 769)

#Figure 8: How many versions/systems have been built over the years?
# save figure with width=656, height=484
df.vpy <- df.g1[,c("VersionKey","BuildYear")] %>% unique(.) %>% group_by(BuildYear) %>% dplyr::summarise(., versions=n())
df.svpy <- merge(df.vpy, df.spy, by.x="BuildYear", by.y="bymin") %>% melt(., "BuildYear")
svpy_plot <-ggplot(df.svpy[df.svpy$BuildYear<=2014,], aes(x = BuildYear, y=value, color=as.factor(variable), group=variable))  + 
  geom_line() +
  geom_line(data=df.svpy[df.svpy$BuildYear>2013,], linetype="dashed") +
  geom_point(data=df.svpy, aes(color=variable))+
  scale_color_brewer(palette="Paired") +
  labs(x = "Build year", y = "Number") +
  jp1theme+
  theme(legend.title=element_blank(),legend.position = c(.03,.95))+
  scale_x_continuous(breaks=c(2000,2005,2010,2015))
  ggtitle("Number of Versions and Systems Built each Year between 2000 and 2016")
svpy_plot
dev.print(png, file = "output/figure8.png", width = 656, height = 484)

#Figure 9: How have specific systems developed over time?
#width=433, height=576
df.vpspy <- df.g1[,c("SystemName","VersionKey","BuildYear")] %>% unique(.) %>% group_by(SystemName) %>% mutate(., versions=n()) %>% .[which(.[,"versions"] > 4),] %>% group_by_("SystemName","BuildYear") %>% dplyr::summarise(., versions=n())
vpspy_plot <- ggplot(df.vpspy, alpha = 0.2, aes(x = BuildYear,y=versions))  + 
  geom_line(aes(group=SystemName)) +
  geom_point() + 
  facet_wrap(~SystemName, ncol=2) +
  labs(x = "Build year", y = "Number of versions") +
  jp1theme +
  theme(panel.spacing = unit(1, "lines"), 
        strip.text.x = element_text(margin = margin(.1, 0, .1, 0, "cm"))) +
  scale_color_brewer(palette="Blues") +
  ggtitle("Number of Versions of Systems with >4 Versions Built per Year")
vpspy_plot
dev.print(png, file = "output/figure9.png", width = 433, height = 576)

#Multimedia Appendix 7: table showing ranking by evidence base
df.sps <- df.g2[,c("SystemKey","EvaluationKey","Quality")] %>% unique(.) %>% group_by(SystemKey,Quality) %>% dplyr::summarise(., evalcount=n())
df.ppe <- df.g2[,c("SystemKey","EvaluationKey","ArmKey","Quality","Npre","Npost")] %>% unique(.) %>% group_by(SystemKey, Quality) %>% dplyr::summarise(., npartpre=sum(Npre,na.rm = TRUE),npartpost=sum(Npost, na.rm=TRUE))
#merge the two dataframes plus the df.vps dataframe to have the version count and names as well
df.qualityranking <- merge(df.vps,df.sps, by="SystemKey", all.y = TRUE)
df.qualityranking <- merge(df.qualityranking, df.ppe, by=c("SystemKey","Quality"))
df.qualityranking_a <- df.qualityranking[which(df.qualityranking$Quality==1),]
df.qualityranking_a <- df.qualityranking_a[order(-df.qualityranking_a$evalcount,-df.qualityranking_a$npartpre, -df.qualityranking_a$npartpost),]
df.qualityranking_a$rank <- ave(order(-df.qualityranking_a$evalcount, -df.qualityranking_a$npartpre, -df.qualityranking_a$npartpost), -df.qualityranking_a$evalcount, -df.qualityranking_a$npartpre, -df.qualityranking_a$npartpost) 
df.qualityranking_a <- df.qualityranking_a[order(df.qualityranking_a$rank),]
write.table(df.qualityranking_a, "output/mma7a.csv", sep=";", row.names=FALSE)

df.qualityranking_b <- df.qualityranking[which(df.qualityranking$Quality==0),]
df.qualityranking_b <- df.qualityranking_b[order(-df.qualityranking_b$evalcount,-df.qualityranking_b$npartpre, -df.qualityranking_b$npartpost),]
df.qualityranking_b$rank <- ave(order(-df.qualityranking_b$evalcount, -df.qualityranking_b$npartpre, -df.qualityranking_b$npartpost), -df.qualityranking_b$evalcount, -df.qualityranking_b$npartpre, -df.qualityranking_b$npartpost) 
df.qualityranking_b <- df.qualityranking_b[order(df.qualityranking_b$rank),]
write.table(df.qualityranking_b, "output/mma7b.csv", sep=";", row.names=FALSE)



